package ltd.llvy.sensitive.support;

import cn.hutool.core.util.DesensitizedUtil;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Objects;

import ltd.llvy.sensitive.annotation.Desensitization;
import ltd.llvy.sensitive.filter.DesensitizedFilter;

/**
 * (JsonDesensitizationSerializer)
 *
 * @author zweiandlen
 * @since 2024/9/27 14:35
 */
public class JsonDesensitizationSerializer extends JsonSerializer<String>
    implements ContextualSerializer {

  /** 脱敏字段类型 */
  private DesensitizedUtil.DesensitizedType type;

  /** 过滤方法. */
  private Class<?> clazz;

  public JsonDesensitizationSerializer() {}

  public JsonDesensitizationSerializer(DesensitizedUtil.DesensitizedType type, Class<?> clazz) {
    this.type = type;
    this.clazz = clazz;
  }

  @Override
  public void serialize(
      String plaintext, JsonGenerator jsonGenerator, SerializerProvider serializerProvider)
      throws IOException {
    String desensitize = DesensitizedUtil.desensitized(plaintext, type);
    DesensitizedFilter filter = null;
    try {
      filter = (DesensitizedFilter) clazz.getConstructor().newInstance();
    } catch (InstantiationException
        | IllegalAccessException
        | InvocationTargetException
        | NoSuchMethodException e) {
      throw new RuntimeException(e);
    }
    // 符合过滤条件的显示明文，不符合的脱敏
    if (filter.doFilter()) {
      jsonGenerator.writeString(plaintext);
    } else {
      jsonGenerator.writeString(desensitize);
    }
  }

  @Override
  public JsonSerializer<?> createContextual(
      SerializerProvider serializerProvider, BeanProperty property) throws JsonMappingException {
    if (property != null) {
      // 判断是不是String类型的参数
      if (Objects.equals(property.getType().getRawClass(), String.class)) {
        // 获取属性上面的注解
        Desensitization annotation = property.getAnnotation(Desensitization.class);
        if (annotation == null) {
          annotation = property.getContextAnnotation(Desensitization.class);
        }
        if (annotation != null) {
          // 给当前属性赋值
          return new JsonDesensitizationSerializer(annotation.type(), annotation.filter());
        }
      }
      return serializerProvider.findValueSerializer(property.getType(), property);
    } else {
      return serializerProvider.findNullValueSerializer(null);
    }
  }
}
